==> lms-0.0.x-dev.tar.gz: lms/opigno_learning_path/src/ThemeHelper.php <==
<?php

declare(strict_types=1);

namespace Drupal\opigno_learning_path;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Link;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\grequest\MembershipRequestManager;
use Drupal\grequest\Plugin\GroupContentEnabler\GroupMembershipRequest;
use Drupal\opigno_learning_path\Entity\Bundle\LearningPath;
use Drupal\opigno_learning_path\Entity\LPStatus;

/**
 * Theme helper service for learning paths.
 */
final class ThemeHelper implements TrustedCallbackInterface {

  use StringTranslationTrait;

  private TrainingManager $trainingManager;
  private AccountInterface $currentUser;
  private MembershipRequestManager $membershipRequestManager;
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * The constructor.
   */
  public function __construct(
    TrainingManager $trainingManager,
    AccountInterface $currentUser,
    MembershipRequestManager $membershipRequestManager,
    EntityTypeManagerInterface $entityTypeManager
  ) {
    $this->trainingManager = $trainingManager;
    $this->currentUser = $currentUser;
    $this->membershipRequestManager = $membershipRequestManager;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Get lazy builder element for the training action link.
   *
   * @return mixed[]
   *   Renderable array.
   */
  public static function lazyGetUserActionElement(LearningPath $group): array {
    $build = [
      '#lazy_builder' => [
        'learning_path.theme_helper:getUserActionElement',
        [$group->id()],
      ],
    ];
    return $build;
  }

  /**
   * Get learning path action link or info box.
   *
   * @return mixed[]
   *   Renderable array.
   */
  public function getUserActionElement(string $group_id, string $html_classes = ''): array {
    $html_classes = \explode(' ', $html_classes);
    $group = $this->entityTypeManager->getStorage('group')->load($group_id);
    \assert($group instanceof LearningPath);

    $cacheability = new CacheableMetadata();
    $cacheability->addCacheableDependency($group);

    $cache_contexts = [
      'user.roles:authenticated',
      'route.group',
    ];
    $build = [];
    $cacheability->addCacheTags(['group.classes:' . $group_id]);

    if ($this->currentUser->isAnonymous()) {
      $url = Url::fromRoute(
        'user.login',
        ['destination' => $group->toUrl()->toString()],
        ['attributes' => ['class' => $html_classes + ['join-link']]],
      );
      $build = $this->buildLink($this->t('Login / Register'), $url);
      $cacheability->addCacheContexts($cache_contexts);
      $cacheability->applyTo($build);
      return $build;
    }

    $take_access = $group->takeAccess($this->currentUser);
    $cacheability->addCacheableDependency($take_access);
    if (!$take_access->isAllowed()) {
      $classes = $group->getClasses();
      if (\count($classes) === 0) {
        $build = $this->buildInfo('no-classes', $this->t('No classes available'));
      }
      elseif (!$group->requiresValidation()) {
        $url = Url::fromRoute(
          'entity.group.join',
          ['group' => $group->id()],
          [
            'attributes' => [
              'class' => $html_classes + ['join-link'],
              'data-toggle' => 'modal',
              'data-target' => '#join-group-form-overlay',
            ],
          ],
        );
        $build = $this->buildLink($this->t('Enroll'), $url);
      }
      else {
        // Subscribe to the default class from learning path page to simplify.
        $user = $this->entityTypeManager->getStorage('user')->load($this->currentUser->id());
        $default_class = \reset($classes);
        $membership_request = $this->membershipRequestManager->getMembershipRequest($user, $default_class);
        $request_status = NULL;
        if ($membership_request !== NULL) {
          $request_status = $membership_request->get(GroupMembershipRequest::STATUS_FIELD)->value;
        }

        if ($request_status === GroupMembershipRequest::REQUEST_PENDING) {
          $build = $this->buildInfo('approval', $this->t('Approval Pending'));
        }
        elseif ($request_status === GroupMembershipRequest::REQUEST_REJECTED) {
          $build = $this->buildInfo('rejected', $this->t('Subscription rejected'));
        }
        else {
          // Next: this needs to be a custom route where class can be selected.
          $url = Url::fromRoute(
            'entity.group.group_request_membership',
            ['group' => $group->id()],
            [
              'attributes' => [
                'class' => $html_classes + ['join-link'],
                'data-toggle' => 'modal',
                'data-target' => '#join-group-form-overlay',
              ],
            ],
          );
          $build = $this->buildLink($this->t('Request subscription'), $url);
        }
      }
    }
    else {
      $lp_status = $this->trainingManager->getLearningPathStatus($group, $this->currentUser);
      $status = NULL;
      if ($lp_status !== NULL) {
        $status = $lp_status->getStatus();
        $cacheability->addCacheableDependency($lp_status);
      }
      $status = $lp_status === NULL ? NULL : $lp_status->getStatus();

      $url = Url::fromRoute(
        'opigno_learning_path.steps.start',
        ['group' => $group->id()],
        ['attributes' => ['class' => $html_classes + ['start-link']]],
      );
      if ($status === NULL) {
        $build = $this->buildLink($this->t('Start'), $url);
      }
      elseif ($status === LPStatus::STATUS_PROGRESS) {
        $build = $this->buildLink($this->t('Continue'), $url);
      }
      elseif ($status === LPStatus::STATUS_FAILED || $status === LPStatus::STATUS_PASSED) {
        if ($group->noFailMode()) {
          $build = $this->buildLink($this->t('Revisit'), $url);
        }
        else {
          // @todo Add a confirmation step when restarting.
          $build = $this->buildLink($this->t('Restart'), $url);
        }
      }
      elseif ($status === LPStatus::STATUS_NEEDS_EVALUATION) {
        $build = $this->buildInfo('needs-evaluation', $this->t('Awaits grading'));
      }
    }

    $cacheability->addCacheContexts($cache_contexts);
    $cacheability->applyTo($build);

    return $build;
  }

  /**
   * Code saver.
   *
   * @return mixed[]
   *   Renderable array.
   */
  private function buildInfo(string $class, TranslatableMarkup $info): array {
    return [
      '#theme' => 'learning_path_action_info',
      '#class' => $class,
      '#info_text' => $info,
    ];
  }

  /**
   * Code saver.
   *
   * @return mixed[]
   *   Renderable array.
   */
  private function buildLink(TranslatableMarkup $text, Url $url) {
    $link = Link::fromTextAndUrl($text, $url);
    return [
      'link' => $link->toRenderable(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return [
      'getUserActionElement',
    ];
  }

}

==> lms-0.2.x-dev.tar.gz: lms/opigno_learning_path/src/ThemeHelper.php <==
<?php

declare(strict_types=1);

namespace Drupal\opigno_learning_path;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Link;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\grequest\MembershipRequestManager;
use Drupal\grequest\Plugin\Group\Relation\GroupMembershipRequest;
use Drupal\opigno_learning_path\Entity\Bundle\LearningPath;
use Drupal\opigno_learning_path\Entity\LPStatus;

/**
 * Theme helper service for learning paths.
 */
final class ThemeHelper implements TrustedCallbackInterface {

  use StringTranslationTrait;

  private TrainingManager $trainingManager;
  private AccountInterface $currentUser;
  private MembershipRequestManager $membershipRequestManager;
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * The constructor.
   */
  public function __construct(
    TrainingManager $trainingManager,
    AccountInterface $currentUser,
    MembershipRequestManager $membershipRequestManager,
    EntityTypeManagerInterface $entityTypeManager
  ) {
    $this->trainingManager = $trainingManager;
    $this->currentUser = $currentUser;
    $this->membershipRequestManager = $membershipRequestManager;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Get lazy builder element for the training action link.
   *
   * @return mixed[]
   *   Renderable array.
   */
  public static function lazyGetUserActionElement(LearningPath $group): array {
    $build = [
      '#lazy_builder' => [
        'learning_path.theme_helper:getUserActionElement',
        [$group->id()],
      ],
    ];
    return $build;
  }

  /**
   * Get learning path action link or info box.
   *
   * @return mixed[]
   *   Renderable array.
   */
  public function getUserActionElement(string $group_id, string $html_classes = ''): array {
    $html_classes = \explode(' ', $html_classes);
    $group = $this->entityTypeManager->getStorage('group')->load($group_id);
    \assert($group instanceof LearningPath);

    $cacheability = new CacheableMetadata();
    $cacheability->addCacheableDependency($group);

    $cache_contexts = [
      'user.roles:authenticated',
      'route.group',
    ];
    $build = [];
    $cacheability->addCacheTags(['group.classes:' . $group_id]);

    if ($this->currentUser->isAnonymous()) {
      $url = Url::fromRoute(
        'user.login',
        ['destination' => $group->toUrl()->toString()],
        ['attributes' => ['class' => $html_classes + ['join-link']]],
      );
      $build = $this->buildLink($this->t('Login / Register'), $url);
      $cacheability->addCacheContexts($cache_contexts);
      $cacheability->applyTo($build);
      return $build;
    }

    $take_access = $group->takeAccess($this->currentUser);
    $cacheability->addCacheableDependency($take_access);
    if (!$take_access->isAllowed()) {
      $classes = $group->getClasses();
      if (\count($classes) === 0) {
        $build = $this->buildInfo('no-classes', $this->t('No classes available'));
      }
      elseif (!$group->requiresValidation()) {
        $url = Url::fromRoute(
          'entity.group.join',
          ['group' => $group->id()],
          [
            'attributes' => [
              'class' => $html_classes + ['join-link'],
              'data-toggle' => 'modal',
              'data-target' => '#join-group-form-overlay',
            ],
          ],
        );
        $build = $this->buildLink($this->t('Enroll'), $url);
      }
      else {
        // Subscribe to the default class from learning path page to simplify.
        $user = $this->entityTypeManager->getStorage('user')->load($this->currentUser->id());
        $membership_request = $this->membershipRequestManager->getMembershipRequest($user, $group);
        $request_status = NULL;
        if ($membership_request !== NULL) {
          $request_status = $membership_request->get(GroupMembershipRequest::STATUS_FIELD)->value;
        }

        if ($request_status === GroupMembershipRequest::REQUEST_PENDING) {
          $build = $this->buildInfo('approval', $this->t('Approval Pending'));
        }
        elseif ($request_status === GroupMembershipRequest::REQUEST_REJECTED) {
          $build = $this->buildInfo('rejected', $this->t('Subscription rejected'));
        }
        else {
          // Next: this needs to be a custom route where class can be selected.
          $url = Url::fromRoute(
            'entity.group.group_request_membership',
            ['group' => $group->id()],
            [
              'attributes' => [
                'class' => $html_classes + ['join-link'],
                'data-toggle' => 'modal',
                'data-target' => '#join-group-form-overlay',
              ],
            ],
          );
          $build = $this->buildLink($this->t('Request subscription'), $url);
        }
      }
    }
    else {
      $lp_status = $this->trainingManager->getLearningPathStatus($group, $this->currentUser);
      $status = NULL;
      if ($lp_status !== NULL) {
        $status = $lp_status->getStatus();
        $cacheability->addCacheableDependency($lp_status);
      }
      $status = $lp_status === NULL ? NULL : $lp_status->getStatus();

      $url = Url::fromRoute(
        'opigno_learning_path.steps.start',
        ['group' => $group->id()],
        ['attributes' => ['class' => $html_classes + ['start-link']]],
      );
      if ($status === NULL) {
        $build = $this->buildLink($this->t('Start'), $url);
      }
      elseif ($status === LPStatus::STATUS_PROGRESS) {
        $build = $this->buildLink($this->t('Continue'), $url);
      }
      elseif ($status === LPStatus::STATUS_FAILED || $status === LPStatus::STATUS_PASSED) {
        if ($group->noFailMode()) {
          $build = $this->buildLink($this->t('Revisit'), $url);
        }
        else {
          $module_ids = opigno_learning_path_get_modules_ids_by_group($group);
          $modules = $this->entityTypeManager->getStorage('opigno_module')->loadMultiple($module_ids);
          $allowed_attempts = 0;
          foreach ($modules as $module) {
            if ((int) $module->takes->value == 1) {
              $allowed_attempts = 1;
            }
          }
          if ($allowed_attempts === 1) {
            $build = $this->buildInfo('needs-evaluation', $this->t('Awaits grading'));
          }
          else {
            // @todo Add a confirmation step when restarting.
            $build = $this->buildLink($this->t('Restart'), $url);
          }
        }
      }
      elseif ($status === LPStatus::STATUS_NEEDS_EVALUATION) {
        $build = $this->buildInfo('needs-evaluation', $this->t('Awaits grading'));
      }
    }

    $cacheability->addCacheContexts($cache_contexts);
    $cacheability->applyTo($build);

    return $build;
  }

  /**
   * Code saver.
   *
   * @return mixed[]
   *   Renderable array.
   */
  private function buildInfo(string $class, TranslatableMarkup $info): array {
    return [
      '#theme' => 'learning_path_action_info',
      '#class' => $class,
      '#info_text' => $info,
    ];
  }

  /**
   * Code saver.
   *
   * @return mixed[]
   *   Renderable array.
   */
  private function buildLink(TranslatableMarkup $text, Url $url) {
    $link = Link::fromTextAndUrl($text, $url);
    return [
      'link' => $link->toRenderable(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return [
      'getUserActionElement',
    ];
  }

}
